#include <mysql/my_global.h>
#include <mysql/my_sys.h>
#include <mysql/mysql.h>
#include <mysql/m_ctype.h>
#include <mysql/m_string.h>
   
#define MAX_IMAGE_SIZE 1048576
#define MAX_FILE_NAME 1024

my_bool getImage_init(UDF_INIT *initid, UDF_ARGS *args, char *message) {
	if (args->arg_count != 1) {
		strmov(message,"Usage: getImage( fileName )");
		return 1;
	}

	args->arg_type[0] = STRING_RESULT;

	if ( !(initid->ptr = 
		(char *) malloc( sizeof(char) * MAX_IMAGE_SIZE ) ) ) {
		strmov(message, "Couldn't allocate memory!");
		return 1;
	}
	bzero( initid->ptr, sizeof(char) * MAX_IMAGE_SIZE );
	return 0;
}

void getImage_deinit(UDF_INIT *initid) {
	if (initid->ptr)
		free(initid->ptr);
}

char *getImage(UDF_INIT *initid, UDF_ARGS *args, char *result,
	       unsigned long *length, char *is_null, char *error) {
	FILE *img_file;
	size_t img_size;
	char filename[MAX_FILE_NAME];
	char old_dir[MAX_FILE_NAME];

	bzero(filename, MAX_FILE_NAME);
	bzero(old_dir, MAX_FILE_NAME);

	getcwd(old_dir, MAX_FILE_NAME);
	chdir("/home/ryarger/pictures");
	strncpy( filename, args->args[0], args->lengths[0] );
	img_file = fopen( filename, "r" );
	if (! img_file ) {
		*is_null = 1;
		return 0;	
	}
	
	img_size =
		fread( (void *)initid->ptr, sizeof(char),
			MAX_IMAGE_SIZE, img_file );
	
	*length = (unsigned long)img_size;
	chdir(old_dir);
	fclose(img_file);
	return initid->ptr;
}

struct point {
	double x;
	double y;
	long count;
};

my_bool distanceFromCenter_init( UDF_INIT* initid, UDF_ARGS* args, 
	char* message ) {

	if (args->arg_count != 2) {
		strmov(message,"Usage: distanceFromCenter( x, y )");
		return 1;
	}

	args->arg_type[0] = REAL_RESULT;
	args->arg_type[1] = REAL_RESULT;

	initid->maybe_null = 0;
	initid->decimals = 5;
	initid->max_length = 10;

	initid->ptr = malloc( sizeof( struct point ) );
	return 0;
}

void distanceFromCenter_deinit( UDF_INIT* initid ) {
	free(initid->ptr);
}

void distanceFromCenter_add( UDF_INIT* initid, UDF_ARGS* args, 
	char* is_null, char *error ) {
	((struct point *)initid->ptr)->x += *((double *)args->args[0]);
	((struct point *)initid->ptr)->y += *((double *)args->args[0]);
	fprintf(stderr, "Adding: X: %f Y: %f\n", 
		*((double *)args->args[0]),
		*((double *)args->args[0]) );
	((struct point *)initid->ptr)->count++;
}

void distanceFromCenter_reset( UDF_INIT* initid, UDF_ARGS* args, 
	char* is_null, char *error ) {
	((struct point *)initid->ptr)->x = 0.0;
	((struct point *)initid->ptr)->y = 0.0;
	((struct point *)initid->ptr)->count = 0;
	distanceFromCenter_add( initid, args, is_null, error );
}

double distanceFromCenter( UDF_INIT* initid, UDF_ARGS* args, 
	char* is_null, char *error ) {
	double avg_x, avg_y;

	avg_x = ((struct point *)initid->ptr)->x / 
		(double)((struct point *)initid->ptr)->count;
	avg_y = ((struct point *)initid->ptr)->y / 
		(double)((struct point *)initid->ptr)->count;
	
	fprintf(stderr, "X: %f Y: %f\n", avg_x, avg_y);
	return sqrt( avg_x * avg_x + avg_y + avg_y );
}
